/**
 * 
 */
package eu.etaxonomy.taxeditor.navigation.key.polytomous;

import java.util.List;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IPolytomousKeyService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.ContextListenerAdapter;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 *
 */
public class PolytomousKeyViewPart extends ViewPart implements
		IConversationEnabled, IPostOperationEnabled {

	private class FilterModifyListener implements ModifyListener{
		@Override
		public void modifyText(ModifyEvent e) {
			ViewerFilter filter = new ViewerFilter(){

				@Override
				public boolean select(Viewer viewer, Object parentElement,
						Object element) {
					
					if(element instanceof PolytomousKey){
						PolytomousKey key = (PolytomousKey) element;
						if(key.getTitleCache().contains(text_filter.getText())){
							return true;
						}
					}
					
					return false;
				}
				
			};
			
			viewer.setFilters(new ViewerFilter[]{filter});
		}
	}
	
	
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.key.polytomous.polytomousKeyViewPart";

	public static final String OPEN_COMMAND_ID = "eu.etaxonomy.taxeditor.navigation.key.polytomous.edit";
	
	private class ContextListener extends ContextListenerAdapter{
		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		public void contextStop(IMemento memento, IProgressMonitor monitor) {
			monitor.subTask("Shutting down Polytomous Key Viewer");
			viewer.setInput(null);
		}
		
		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.ContextListenerAdapter#contextStart(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		public void contextStart(IMemento memento, IProgressMonitor monitor) {
			monitor.subTask("Initiating Polytomous Key Viewer");
			setInput();
		}
	}
	
	private ListViewer viewer;
	private ConversationHolder conversation;
	private IContextListener contextListener;
	private final CdmFormFactory formFactory;
	private Text text_filter;

	private PolytomousKeyViewPartDataChangeBehavior dataChangeBehavior;

	public PolytomousKeyViewPart(){
		formFactory = new CdmFormFactory(Display.getDefault());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {

		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout());
		
		contextListener = new ContextListener();
		CdmStore.getContextManager().addContextListener(contextListener);
		
		text_filter = formFactory.createText(container, "");
		text_filter.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		text_filter.addModifyListener(new FilterModifyListener());
		
		viewer = new ListViewer(container, SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
		viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		viewer.setContentProvider(new PolytomousKeyContentProvider());
		viewer.setLabelProvider(new PolytomousKeyLabelProvider());
		
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			
			@Override
			public void doubleClick(DoubleClickEvent event) {
				
				ICommandService commandService = (ICommandService)getSite().getService(ICommandService.class);
				
				Command command = commandService.getCommand(OPEN_COMMAND_ID); 
				if(command.isEnabled()) {
					IHandlerService handlerService = (IHandlerService)getSite().getService(IHandlerService.class);
					try {
						handlerService.executeCommand(OPEN_COMMAND_ID, null);
					} catch (NotDefinedException e) {
						throw new RuntimeException("Could not find open command: " + OPEN_COMMAND_ID);
					} catch (Exception e) {
						NavigationUtil.error(getClass(), "An exception occured while trying to open a selection", e);
					}
				}
			}
		});
		
		getSite().setSelectionProvider(viewer);
				
		createMenu();
		
		if(CdmStore.isActive()){
			setInput();
		}
	}
	
	private void createMenu(){
		MenuManager menuManager = new MenuManager();
		menuManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		
		getSite().registerContextMenu(menuManager, viewer);
				
		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);
		
		control.setMenu(menu);	
	}

	private void setInput() {
		conversation = CdmStore.createConversation();
		conversation.registerForDataStoreChanges(this);
		List<PolytomousKey> input = CdmStore.getService(IPolytomousKeyService.class).list(PolytomousKey.class, null, null, null, null);			
		viewer.setInput(input);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap changeEvents) {
		if (dataChangeBehavior == null) {
			dataChangeBehavior = new PolytomousKeyViewPartDataChangeBehavior(this);
		}

		DataChangeBridge.handleDataChange(changeEvents, dataChangeBehavior);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		conversation.unregisterForDataStoreChanges(this);
		conversation.close();
		CdmStore.getContextManager().removeContextListener(contextListener);
		super.dispose();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		viewer.refresh();
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		return true;
	}

	/**
	 * 
	 */
	public void refresh() {
		getConversationHolder().bind();
		List<PolytomousKey> input = CdmStore.getService(IPolytomousKeyService.class).list(PolytomousKey.class, null, null, null, null);			
		viewer.setInput(input);
	}

}
