// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.navigator.ILinkHelper;

import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * When a taxon is open in an editor and this editor has focus, its entry in the
 * navigator's taxonomic tree is selected.
 *
 * @author p.ciardelli
 * @created 03.06.2009
 * @version 1.0
 */
public class TaxonLinkHelper implements ILinkHelper {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.navigation.taxon"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.taxonlinkhelper"; //$NON-NLS-1$
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.ILinkHelper#activateEditor(org.eclipse.ui.IWorkbenchPage, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	/** {@inheritDoc} */
	public void activateEditor(IWorkbenchPage page,
			IStructuredSelection selection) {
		try {
			if (selection == null || selection.isEmpty()) {
				return;
			}
			if (selection.getFirstElement() instanceof TaxonNode) {
				TaxonNode taxonNode = (TaxonNode) selection.getFirstElement();
				TaxonEditorInput taxonEditorInput;
				
					taxonEditorInput = TaxonEditorInput.NewInstance(taxonNode.getUuid());
				
				IEditorPart editor = null;
				if ((editor = page.findEditor(taxonEditorInput)) != null) {
					page.bringToTop(editor);
				}
			}
		} catch (Exception e) {
			EditorUtil.warningDialog("Could not create Taxon", this, e.getMessage());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.ILinkHelper#findSelection(org.eclipse.ui.IEditorInput)
	 */
	/** {@inheritDoc} */
	public IStructuredSelection findSelection(IEditorInput editorInput) {
		if (editorInput instanceof TaxonEditorInput) {
			TaxonNode taxonNode = ((TaxonEditorInput) editorInput).getTaxonNode();
			if (taxonNode != null) {
				return new StructuredSelection(taxonNode);
			}
		}
		return StructuredSelection.EMPTY;
	}
}
