// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.navigator.IDescriptionProvider;
import org.hibernate.LazyInitializationException;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;

/**
 * <p>TaxonNodeLabelProvider class.</p>
 *
 * @author p.ciardelli
 * @created 04.06.2009
 * @version 1.0
 */
public class TaxonNodeLabelProvider extends ColumnLabelProvider 
		implements ILabelProvider, IDescriptionProvider, IStyledLabelProvider {
	
	/** {@inheritDoc} */
	@Override
	public Image getImage(Object element) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getText(Object element) {
		if (element instanceof TaxonNode){
			TaxonNode taxonNode = (TaxonNode) HibernateProxyHelper.deproxy(element);
			
			try{
				Taxon taxon = (Taxon) HibernateProxyHelper.deproxy(taxonNode.getTaxon());
				if(taxon == null){
					NavigationUtil.error(getClass(), "The taxon node does not have a taxon attached to it. Please check your import. TaxonNode: " + element, null);
//					throw new RuntimeException("The data your are trying to view is corrupt. Please check your import.");
				}else{
					try{					
						return taxon.getName() != null ? ((IIdentifiableEntity) HibernateProxyHelper.deproxy(taxon.getName())).getTitleCache() : new String();
					}catch(Exception e){
						NavigationUtil.error(getClass(), e);
					}
				}
			}catch (LazyInitializationException e){
				NavigationUtil.error(getClass(), e);
			}
			
		}
		return new String();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.IDescriptionProvider#getDescription(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getDescription(Object anElement) {
		if (anElement instanceof TaxonNode) {
			Taxon data = ((TaxonNode) anElement).getTaxon();
			return "Taxon: " + data.getTitleCache(); //$NON-NLS-1$
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider#getStyledText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public StyledString getStyledText(Object element) {
		return new StyledString(getText(element));
	}
}
