// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.MoveTaxonOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>TaxonNodeDropAdapterAssistant class.</p>
 *
 * @author p.ciardelli
 * @created 03.06.2009
 * @version 1.0
 */
public class TreeNodeDropAdapterAssistant extends CommonDropAdapterAssistant implements IPostOperationEnabled{
	private static final Logger logger = Logger
			.getLogger(TreeNodeDropAdapterAssistant.class);

	/** Constant <code>ID="eu.etaxonomy.taxeditor.navigation.navig"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.navigator.dropassistant"; //$NON-NLS-1$
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#handleDrop(org.eclipse.ui.navigator.CommonDropAdapter, org.eclipse.swt.dnd.DropTargetEvent, java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus handleDrop(CommonDropAdapter dropAdapter,
			DropTargetEvent dropTargetEvent, Object target) {
			
		if (target instanceof ITreeNode) {
			Set<TaxonNode> taxonNodes = getSelectedTaxa();
			ITreeNode targetTreeNode = (ITreeNode) target;
			if(taxonNodes != null)
				return moveTaxon(taxonNodes, targetTreeNode);
		}
							
		return Status.CANCEL_STATUS;
	}
	
	private Set<TaxonNode> getSelectedTaxa(){
		HashSet<TaxonNode> taxonNodes = new HashSet<TaxonNode>();		
		
		ISelection selection = LocalSelectionTransfer.getTransfer().getSelection();
		if (selection instanceof TreeSelection) {
		
			Iterator selectionIterator = ((TreeSelection) selection).iterator();
				
			while (selectionIterator.hasNext()){
				Object object = selectionIterator.next();
				if(object instanceof TaxonNode){
					TaxonNode taxonNode = (TaxonNode) object;
					taxonNodes.add(taxonNode);
				}
			}
		}
		return taxonNodes.size() > 0 ? taxonNodes : null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus validateDrop(Object target, int operation,
			TransferData transferType) {	
		if (target instanceof ITreeNode) {
			// do not allow to drop onto itself
			for(TaxonNode taxonNode : getSelectedTaxa()){
				if (taxonNode.equals(target)) {
					return Status.CANCEL_STATUS;
				}
			}
			return Status.OK_STATUS;
		}		
		return Status.CANCEL_STATUS;
	}
	

	/**
	 * @param childTaxonNode
	 * @param parentTaxon
	 * @return
	 */
	private IStatus moveTaxon(Set<TaxonNode> taxonNodes, ITreeNode targetITreeNode) {
		
		TaxonNavigator taxonNavigator;
		taxonNavigator = (TaxonNavigator) NavigationUtil.showView(TaxonNavigator.ID);
		
		if(targetITreeNode instanceof TaxonNode){
		
			TaxonNode targetTaxonNode = (TaxonNode) targetITreeNode;
			
//			for(TaxonNode taxonNode : taxonNodes){
//				if (taxonNode.equals(targetTaxonNode)) {
//					return Status.CANCEL_STATUS;
//				}
//			}
			
			// Make sure parent taxon does not have unsaved changes
			if (NavigationUtil.isDirty(targetTaxonNode)){
				MessageDialog.openWarning(NavigationUtil.getShell(), "Unsaved Parent Taxon", "There are unsaved " +
				"changes in the parent taxon. Pleas save first.");
				return Status.CANCEL_STATUS;
			}
			
			
			// Make sure parentTaxon is not the drop target
//			if (!childTaxonNode.isTopmostNode() && childTaxonNode.getParent().equals(targetTaxonNode)){
//				return Status.CANCEL_STATUS;
//			}
			
			// Make sure taxon is not being dropped onto itself
//			if (childTaxonNode.equals(targetTaxonNode)) {
//				return Status.CANCEL_STATUS;
//			}
			
			

		}	
		
		IUndoContext workspaceUndoContext = NavigationUtil.getWorkbenchUndoContext();
		if (workspaceUndoContext == null) {
			logger.error("Workspace undo context is null. DND operation cancelled");
			return Status.CANCEL_STATUS;
		}

		AbstractPostOperation operation = new MoveTaxonOperation
				("Move Taxon", workspaceUndoContext, taxonNodes, targetITreeNode, this, taxonNavigator);
		NavigationUtil.executeOperation(operation);	
		
		logger.info("Moved taxa to new parent " + targetITreeNode);
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	/** {@inheritDoc} */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		return true;
	}

	/**
	 * <p>onComplete</p>
	 *
	 * @return a boolean.
	 */
	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
